/*
 * Copy me if you can.
 * by 20h
 */

#include <u.h>
#include <libc.h>

void
be_server(int buflen, char *addr)
{
	char *buf, adir[40], ldir[40];
	int dfd, acfd, lcfd;

	acfd = announce(addr, adir);
	if(acfd < 0)
	{
		perror("announce");
		return;
	}

	while(1)
	{
		lcfd = listen(adir, ldir);
		if(lcfd < 0)
		{
			perror("listen");
			return;
		}

		switch(fork())
		{
			case -1:
				perror("fork");
				close(lcfd);
				break;
			case 0:
				dfd = accept(lcfd, ldir);
				if(dfd < 0)
					return;

				buf = malloc(buflen);
				while(read(dfd, buf, buflen) > 0);
				free(buf);

				exits(0);
			default:
				close(lcfd);
				break;
		}
	}
}

void
be_client(int buflen, char *addr, int sec)
{
	char *buf;
	int fd, nbyte;
	long now;

	nbyte = 0;
	now = time(0);

	fd = dial(addr, 0, 0, 0);
	if(fd < 0)
	{
		perror("dial");
		return;
	}

	buf = malloc(buflen);
	while(write(fd, buf, buflen) > 0)
	{
		nbyte += buflen;
		if(time(0) >= now + sec)
		{
			fprint(2, "%d\n", nbyte, sec);
			now = time(0);
			nbyte = 0;
		}
	}

	free(buf);

	return;
}

void
usage(void)
{

	print("usage: [-c] [-s sec] [-b buflen] net!host!port\n");
	return;
}

void
main(int argc, char *argv[])
{
	int buflen, sec;
	char is_client;

	buflen = 1024;
	is_client = '\1';
	sec = 1;

	ARGBEGIN {
		case 'b':
			buflen = atoi(ARGF());
			break;
		case 'c':
			is_client = '\0';
			break;
		case 's':
			sec = atoi(ARGF());
			break;
		default:
			usage();
			exits(0);
	} ARGEND;

	if(argv[0] == nil)
	{
		usage();
		exits(0);
	}

	switch(is_client)
	{
		case '\0':
			be_server(buflen, argv[0]);
			break;
		case '\1':
			be_client(buflen, argv[0], sec);
			break;
		default:
			usage();
			exits(0);
	}

	exits(0);
}
